import 'package:colors/utils/tools.dart';         

class DefaultGameSettings {
  // available game parameters codes
  static const String parameterCodeDifficultyLevel = 'difficultyLevel';
  static const String parameterCodeBoardSize = 'boardSize';
  static const String parameterCodeColorsCount = 'colorsCount';
  static const List<String> availableParameters = [
    parameterCodeDifficultyLevel,
    parameterCodeBoardSize,
    parameterCodeColorsCount,
  ];

  // difficulty level: available values
  static const String difficultyLevelValueEasy = 'easy';
  static const String difficultyLevelValueMedium = 'medium';
  static const String difficultyLevelValueHard = 'hard';
  static const String difficultyLevelValueNightmare = 'nightmare';
  static const List<String> allowedDifficultyLevelValues = [
    difficultyLevelValueEasy,
    difficultyLevelValueMedium,
    difficultyLevelValueHard,
    difficultyLevelValueNightmare,
  ];
  // difficulty level: default value
  static const String defaultDifficultyLevelValue = difficultyLevelValueMedium;

  // board size: available values
  static const String boardSizeValueSmall = 'small';
  static const String boardSizeValueMedium = 'medium';
  static const String boardSizeValueLarge = 'large';
  static const String boardSizeValueExtra = 'extra';
  static const List<String> allowedBoardSizeValues = [
    boardSizeValueSmall,
    boardSizeValueMedium,
    boardSizeValueLarge,
    boardSizeValueExtra,
  ];
  // board size: default value
  static const String defaultBoardSizeValue = boardSizeValueMedium;

  // colors count: available values
  static const String colorsCountValueLow = '5';
  static const String colorsCountValueMedium = '6';
  static const String colorsCountValueHigh = '7';
  static const String colorsCountValueVeryHigh = '8';
  static const List<String> allowedColorsCountValues = [
    colorsCountValueLow,
    colorsCountValueMedium,
    colorsCountValueHigh,
    colorsCountValueVeryHigh,
  ];
  // colors count: default value
  static const String defaultColorsCountValue = colorsCountValueMedium;

  // available values from parameter code
  static List<String> getAvailableValues(String parameterCode) {
    switch (parameterCode) {
      case parameterCodeDifficultyLevel:
        return DefaultGameSettings.allowedDifficultyLevelValues;
      case parameterCodeBoardSize:
        return DefaultGameSettings.allowedBoardSizeValues;
      case parameterCodeColorsCount:
        return DefaultGameSettings.allowedColorsCountValues;
    }

    printlog('Did not find any available value for game parameter "$parameterCode".');
    return [];
  }

  static int getMovesCountLimitDeltaFromLevelCode(String parameterLevel) {
    const Map<String, int> values = {
      difficultyLevelValueEasy: 5,
      difficultyLevelValueMedium: 3,
      difficultyLevelValueHard: 1,
      difficultyLevelValueNightmare: -1,
    };
    return values[parameterLevel] ??
        getMovesCountLimitDeltaFromLevelCode(DefaultGameSettings.defaultDifficultyLevelValue);
  }

  // parameters displayed with assets (instead of painter)
  static List<String> displayedWithAssets = [
    //
  ];
}
