import 'package:equatable/equatable.dart';
import 'package:flutter/material.dart';
import 'package:hydrated_bloc/hydrated_bloc.dart';

import 'package:colors/config/default_game_settings.dart';
import 'package:colors/models/settings/settings_game.dart';

part 'settings_game_state.dart';

class GameSettingsCubit extends HydratedCubit<GameSettingsState> {
  GameSettingsCubit() : super(GameSettingsState(settings: GameSettings.createDefault()));

  void setValues({
    String? difficultyLevel,
    String? boardSize,
    String? colorsCount,
  }) {
    emit(
      GameSettingsState(
        settings: GameSettings(
          difficultyLevel: difficultyLevel ?? state.settings.difficultyLevel,
          parameterSize: boardSize ?? state.settings.parameterSize,
          parameterColorsCount: colorsCount ?? state.settings.parameterColorsCount,
        ),
      ),
    );
  }

  String getParameterValue(String code) {
    switch (code) {
      case DefaultGameSettings.parameterCodeDifficultyLevel:
        return GameSettings.getLevelValueFromUnsafe(state.settings.difficultyLevel);
      case DefaultGameSettings.parameterCodeBoardSize:
        return GameSettings.getSizeValueFromUnsafe(state.settings.parameterSize);
      case DefaultGameSettings.parameterCodeColorsCount:
        return GameSettings.getColorsValueFromUnsafe(state.settings.parameterColorsCount);
    }

    return '';
  }

  void setParameterValue(String code, String value) {
    final String difficultyLevel = (code == DefaultGameSettings.parameterCodeDifficultyLevel)
        ? value
        : getParameterValue(DefaultGameSettings.parameterCodeDifficultyLevel);
    final String boardSize = (code == DefaultGameSettings.parameterCodeBoardSize)
        ? value
        : getParameterValue(DefaultGameSettings.parameterCodeBoardSize);
    final String colorsCount = (code == DefaultGameSettings.parameterCodeColorsCount)
        ? value
        : getParameterValue(DefaultGameSettings.parameterCodeColorsCount);

    setValues(
      difficultyLevel: difficultyLevel,
      boardSize: boardSize,
      colorsCount: colorsCount,
    );
  }

  @override
  GameSettingsState? fromJson(Map<String, dynamic> json) {
    final String difficultyLevel =
        json[DefaultGameSettings.parameterCodeDifficultyLevel] as String;
    final String boardSize = json[DefaultGameSettings.parameterCodeBoardSize] as String;
    final String colorsCount = json[DefaultGameSettings.parameterCodeColorsCount] as String;

    return GameSettingsState(
      settings: GameSettings(
        difficultyLevel: difficultyLevel,
        parameterSize: boardSize,
        parameterColorsCount: colorsCount,
      ),
    );
  }

  @override
  Map<String, dynamic>? toJson(GameSettingsState state) {
    return <String, dynamic>{
      DefaultGameSettings.parameterCodeDifficultyLevel: state.settings.difficultyLevel,
      DefaultGameSettings.parameterCodeBoardSize: state.settings.parameterSize,
      DefaultGameSettings.parameterCodeColorsCount: state.settings.parameterColorsCount,
    };
  }
}
