import 'package:colors/config/default_game_settings.dart';
import 'package:colors/models/game/board.dart';
import 'package:colors/models/settings/settings_game.dart';
import 'package:colors/models/settings/settings_global.dart';
import 'package:colors/utils/tools.dart';

class Game {
  Game({
    // Settings
    required this.gameSettings,
    required this.globalSettings,

    // State
    this.isRunning = false,
    this.isStarted = false,
    this.isFinished = false,
    this.animationInProgress = false,

    // Base data
    required this.board,

    // Game data
    this.maxMovesCount = 0,
    this.movesCount = 0,
    this.progress = 0,
    this.progressTotal = 0,
    this.progressDelta = 0,
    this.gameWon = false,
  });

  // Settings
  final GameSettings gameSettings;
  final GlobalSettings globalSettings;

  // State
  bool isRunning;
  bool isStarted;
  bool isFinished;
  bool animationInProgress;

  // Base data
  final Board board;

  // Game data
  int maxMovesCount;
  int movesCount;
  int progress;
  int progressTotal;
  int progressDelta;
  bool gameWon;

  factory Game.createEmpty() {
    return Game(
      // Settings
      gameSettings: GameSettings.createDefault(),
      globalSettings: GlobalSettings.createDefault(),
      // Base data
      board: Board.createEmpty(),
    );
  }

  factory Game.createNew({
    GameSettings? gameSettings,
    GlobalSettings? globalSettings,
  }) {
    final GameSettings newGameSettings = gameSettings ?? GameSettings.createDefault();
    final GlobalSettings newGlobalSettings = globalSettings ?? GlobalSettings.createDefault();

    final int baseMaxMovesCount =
        (30 * (newGameSettings.boardSize * newGameSettings.colorsCount) / (17 * 6)).round();
    final int deltaMovesCountFromLevel =
        DefaultGameSettings.getMovesCountLimitDeltaFromLevelCode(
            newGameSettings.difficultyLevel);

    return Game(
      // Settings
      gameSettings: newGameSettings,
      globalSettings: newGlobalSettings,
      // State
      isRunning: true,
      // Base data
      board: Board.createRandom(newGameSettings),
      // Game data
      progress: 1,
      progressTotal: newGameSettings.boardSize * newGameSettings.boardSize,
      maxMovesCount: baseMaxMovesCount + deltaMovesCountFromLevel,
    );
  }

  bool get canBeResumed => isStarted && !isFinished;

  void dump() {
    printlog('');
    printlog('## Current game dump:');
    printlog('');
    printlog('$Game:');
    printlog('  Settings');
    gameSettings.dump();
    globalSettings.dump();
    printlog('  State');
    printlog('    isRunning: $isRunning');
    printlog('    isStarted: $isStarted');
    printlog('    isFinished: $isFinished');
    printlog('    animationInProgress: $animationInProgress');
    printlog('  Base data');
    board.dump();
    printlog('  Game data');
    printlog('    maxMovesCount: $maxMovesCount');
    printlog('    movesCount: $movesCount');
    printlog('    progress: $progress');
    printlog('    progressTotal: $progressTotal');
    printlog('    progressDelta: $progressDelta');
    printlog('    gameWon: $gameWon');
    printlog('');
  }

  @override
  String toString() {
    return '$Game(${toJson()})';
  }

  Map<String, dynamic>? toJson() {
    return <String, dynamic>{
      // Settings
      'gameSettings': gameSettings.toJson(),
      'globalSettings': globalSettings.toJson(),
      // State
      'isRunning': isRunning,
      'isStarted': isStarted,
      'isFinished': isFinished,
      'animationInProgress': animationInProgress,
      // Base data
      'board': board.toJson(),
      // Game data
      'maxMovesCount': maxMovesCount,
      'movesCount': movesCount,
      'progress': progress,
      'progressTotal': progressTotal,
      'progressDelta': progressDelta,
      'gameWon': gameWon,
    };
  }
}
