import 'package:colors/config/default_game_settings.dart';
import 'package:colors/utils/tools.dart';     

class GameSettings {
  String difficultyLevel;
  String parameterSize;
  String parameterColorsCount;

  GameSettings({
    required this.difficultyLevel,
    required this.parameterSize,
    required this.parameterColorsCount,
  });

  static String getLevelValueFromUnsafe(String level) {
    if (DefaultGameSettings.allowedDifficultyLevelValues.contains(level)) {
      return level;
    }

    return DefaultGameSettings.defaultDifficultyLevelValue;
  }

  static String getSizeValueFromUnsafe(String size) {
    if (DefaultGameSettings.allowedBoardSizeValues.contains(size)) {
      return size;
    }

    return DefaultGameSettings.defaultBoardSizeValue;
  }

  static String getColorsValueFromUnsafe(String colors) {
    if (DefaultGameSettings.allowedColorsCountValues.contains(colors)) {
      return colors;
    }

    return DefaultGameSettings.defaultColorsCountValue;
  }

  factory GameSettings.createDefault() {
    return GameSettings(
      difficultyLevel: DefaultGameSettings.defaultDifficultyLevelValue,
      parameterSize: DefaultGameSettings.defaultBoardSizeValue,
      parameterColorsCount: DefaultGameSettings.defaultColorsCountValue,
    );
  }

  int getBoardSizeFromParameter(String parameterSize) {
    const Map<String, int> values = {
      DefaultGameSettings.boardSizeValueSmall: 6,
      DefaultGameSettings.boardSizeValueMedium: 10,
      DefaultGameSettings.boardSizeValueLarge: 14,
      DefaultGameSettings.boardSizeValueExtra: 20,
    };
    return values[parameterSize] ??
        getBoardSizeFromParameter(DefaultGameSettings.defaultBoardSizeValue);
  }

  int get boardSize => getBoardSizeFromParameter(parameterSize);
  int get colorsCount => int.parse(parameterColorsCount);

  void dump() {
    printlog('$GameSettings:');
    printlog('  ${DefaultGameSettings.parameterCodeDifficultyLevel}: $difficultyLevel');
    printlog('  ${DefaultGameSettings.parameterCodeBoardSize}: $parameterSize');
    printlog('  ${DefaultGameSettings.parameterCodeColorsCount}: $parameterColorsCount');
    printlog('');
  }

  @override
  String toString() {
    return '$GameSettings(${toJson()})';
  }

  Map<String, dynamic>? toJson() {
    return <String, dynamic>{
      DefaultGameSettings.parameterCodeDifficultyLevel: difficultyLevel,
      DefaultGameSettings.parameterCodeBoardSize: parameterSize,
      DefaultGameSettings.parameterCodeColorsCount: parameterColorsCount,
    };
  }
}
