import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter/material.dart';

import 'package:colors/config/color_theme.dart';
import 'package:colors/cubit/game_cubit.dart';
import 'package:colors/models/game/cell.dart';
import 'package:colors/models/game/game.dart';

class CellWidget extends StatelessWidget {
  const CellWidget({
    super.key,
    required this.cell,
    required this.itemWidth,
  });

  final Cell cell;
  final double itemWidth;

  @override
  Widget build(BuildContext context) {
    return BlocBuilder<GameCubit, GameState>(
      builder: (BuildContext context, GameState gameState) {
        final Game currentGame = gameState.currentGame;

        final ColorScheme colorScheme = Theme.of(context).colorScheme;

        final String skin = currentGame.globalSettings.skin;
        final squareColor = Color(ColorTheme.getColorCode(skin, cell.value));

        return Container(
          decoration: BoxDecoration(
            border: Border.all(
              color: colorScheme.onSurface,
              width: 4,
            ),
          ),
          child: GestureDetector(
            child: Container(
              color: squareColor,
              padding: const EdgeInsets.all(3),
              child: SizedBox.square(
                dimension: itemWidth - 19,
              ),
            ),
            onTap: () {
              if (!currentGame.animationInProgress &&
                  currentGame.board.getFirstCellValue() != cell.value) {
                BlocProvider.of<GameCubit>(context).fillBoardFromFirstCell(cell.value);
              }
            },
          ),
        );
      },
    );
  }
}
