import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter/material.dart';

import 'package:colors/cubit/game_cubit.dart';
import 'package:colors/models/game/game.dart';
import 'package:colors/ui/painters/board_painter.dart';

class GameBoardWidget extends StatelessWidget {
  const GameBoardWidget({super.key});

  @override
  Widget build(BuildContext context) {
    return BlocBuilder<GameCubit, GameState>(
      builder: (BuildContext context, GameState gameState) {
        final Game currentGame = gameState.currentGame;
        final double boardWidth = MediaQuery.of(context).size.width;

        return GestureDetector(
          onTapUp: (details) {
            final double xTap = details.localPosition.dx;
            final double yTap = details.localPosition.dy;
            final int boardSize = currentGame.gameSettings.boardSize;
            final int col = xTap ~/ (boardWidth / boardSize);
            final int row = yTap ~/ (boardWidth / boardSize);
            final int cellValue = currentGame.board.getCellValue(col, row);

            BlocProvider.of<GameCubit>(context).fillBoardFromFirstCell(cellValue);
          },
          child: CustomPaint(
            size: Size(boardWidth, boardWidth),
            willChange: false,
            painter: BoardPainter(
              game: currentGame,
              colorScheme: Theme.of(context).colorScheme,
            ),
            isComplex: true,
          ),
        );
      },
    );
  }
}
